@@ -15,6 +15,7 @@ except ImportError:
 import inspect
 import logging
 import abc
+import os
 
 from git.odict import OrderedDict
 from git.util import LockFile
@@ -22,6 +23,7 @@ from git.compat import (
     string_types,
     FileType,
     defenc,
+    force_text,
     with_metaclass,
     PY3
 )
@@ -79,6 +81,7 @@ def set_dirty_and_flush_changes(non_const_func):
 
     def flush_changes(self, *args, **kwargs):
         rval = non_const_func(self, *args, **kwargs)
+        self._dirty = True
         self.write()
         return rval
     # END wrapper method
@@ -164,7 +167,7 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
     # list of RawConfigParser methods able to change the instance
     _mutating_methods_ = ("add_section", "remove_section", "remove_option", "set")
 
-    def __init__(self, file_or_files, read_only=True):
+    def __init__(self, file_or_files, read_only=True, merge_includes=True):
         """Initialize a configuration reader to read the given file_or_files and to
         possibly allow changes to it by setting read_only False
 
@@ -173,7 +176,13 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
 
         :param read_only:
             If True, the ConfigParser may only read the data , but not change it.
-            If False, only a single file path or file object may be given."""
+            If False, only a single file path or file object may be given. We will write back the changes
+            when they happen, or when the ConfigParser is released. This will not happen if other
+            configuration files have been included
+        :param merge_includes: if True, we will read files mentioned in [include] sections and merge their
+            contents into ours. This makes it impossible to write back an individual configuration file.
+            Thus, if you want to modify a single conifguration file, turn this off to leave the original
+            dataset unaltered when reading it."""
         cp.RawConfigParser.__init__(self, dict_type=OrderedDict)
 
         # Used in python 3, needs to stay in sync with sections for underlying implementation to work
@@ -182,7 +191,9 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
 
         self._file_or_files = file_or_files
         self._read_only = read_only
+        self._dirty = False
         self._is_initialized = False
+        self._merge_includes = merge_includes
         self._lock = None
 
         if not read_only:
@@ -295,7 +306,7 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
                 if mo:
                     # We might just have handled the last line, which could contain a quotation we want to remove
                     optname, vi, optval = mo.group('option', 'vi', 'value')
-                    if vi in ('=', ':') and ';' in optval:
+                    if vi in ('=', ':') and ';' in optval and not optval.strip().startswith('"'):
                         pos = optval.find(';')
                         if pos != -1 and optval[pos - 1].isspace():
                             optval = optval[:pos]
@@ -313,7 +324,6 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
                     if not e:
                         e = cp.ParsingError(fpname)
                     e.append(lineno, repr(line))
-                    print(lineno, line)
                     continue
             else:
                 line = line.rstrip()
@@ -329,6 +339,9 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         if e:
             raise e
 
+    def _has_includes(self):
+        return self._merge_includes and self.has_section('include')
+
     def read(self):
         """Reads the data stored in the files we have been initialized with. It will
         ignore files that cannot be read, possibly leaving an empty configuration
@@ -337,18 +350,25 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         :raise IOError: if a file cannot be handled"""
         if self._is_initialized:
             return
+        self._is_initialized = True
 
-        files_to_read = self._file_or_files
-        if not isinstance(files_to_read, (tuple, list)):
-            files_to_read = [files_to_read]
+        if not isinstance(self._file_or_files, (tuple, list)):
+            files_to_read = [self._file_or_files]
+        else:
+            files_to_read = list(self._file_or_files)
+        # end assure we have a copy of the paths to handle
 
-        for file_object in files_to_read:
-            fp = file_object
+        seen = set(files_to_read)
+        num_read_include_files = 0
+        while files_to_read:
+            file_path = files_to_read.pop(0)
+            fp = file_path
             close_fp = False
+
             # assume a path if it is not a file-object
-            if not hasattr(file_object, "seek"):
+            if not hasattr(fp, "seek"):
                 try:
-                    fp = open(file_object, 'rb')
+                    fp = open(file_path, 'rb')
                     close_fp = True
                 except IOError:
                     continue
@@ -360,8 +380,33 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
                 if close_fp:
                     fp.close()
             # END read-handling
-        # END  for each file object to read
-        self._is_initialized = True
+
+            # Read includes and append those that we didn't handle yet
+            # We expect all paths to be normalized and absolute (and will assure that is the case)
+            if self._has_includes():
+                for _, include_path in self.items('include'):
+                    if not os.path.isabs(include_path):
+                        if not close_fp:
+                            continue
+                        # end ignore relative paths if we don't know the configuration file path
+                        assert os.path.isabs(file_path), "Need absolute paths to be sure our cycle checks will work"
+                        include_path = os.path.join(os.path.dirname(file_path), include_path)
+                    # end make include path absolute
+                    include_path = os.path.normpath(include_path)
+                    if include_path in seen or not os.access(include_path, os.R_OK):
+                        continue
+                    seen.add(include_path)
+                    files_to_read.append(include_path)
+                    num_read_include_files += 1
+                # each include path in configuration file
+            # end handle includes
+        # END for each file object to read
+
+        # If there was no file included, we can safely write back (potentially) the configuration file
+        # without altering it's meaning
+        if num_read_include_files == 0:
+            self._merge_includes = False
+        # end
 
     def _write(self, fp):
         """Write an .ini-format representation of the configuration state in
@@ -370,7 +415,7 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
             fp.write(("[%s]\n" % name).encode(defenc))
             for (key, value) in section_dict.items():
                 if key != "__name__":
-                    fp.write(("\t%s = %s\n" % (key, str(value).replace('\n', '\n\t'))).encode(defenc))
+                    fp.write(("\t%s = %s\n" % (key, self._value_to_string(value).replace('\n', '\n\t'))).encode(defenc))
                 # END if key is not __name__
         # END section writing
 
@@ -379,6 +424,10 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         for name, value in self._sections.items():
             write_section(name, value)
 
+    def items(self, section_name):
+        """:return: list((option, value), ...) pairs of all items in the given section"""
+        return [(k, v) for k, v in super(GitConfigParser, self).items(section_name) if k != '__name__']
+
     @needs_values
     def write(self):
         """Write changes to our file, if there are changes at all
@@ -386,6 +435,19 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         :raise IOError: if this is a read-only writer instance or if we could not obtain
             a file lock"""
         self._assure_writable("write")
+        if not self._dirty:
+            return
+
+        if isinstance(self._file_or_files, (list, tuple)):
+            raise AssertionError("Cannot write back if there is not exactly a single file to write to, have %i files"
+                                 % len(self._file_or_files))
+        # end assert multiple files
+
+        if self._has_includes():
+            log.debug("Skipping write-back of confiuration file as include files were merged in." +
+                      "Set merge_includes=False to prevent this.")
+            return
+        # end
 
         fp = self._file_or_files
         close_fp = False
@@ -421,8 +483,6 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         if self.read_only:
             raise IOError("Cannot execute non-constant method %s.%s" % (self, method_name))
 
-    @needs_values
-    @set_dirty_and_flush_changes
     def add_section(self, section):
         """Assures added options will stay in order"""
         return super(GitConfigParser, self).add_section(section)
@@ -474,6 +534,11 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
 
         return valuestr
 
+    def _value_to_string(self, value):
+        if isinstance(value, (int, float, bool)):
+            return str(value)
+        return force_text(value)
+
     @needs_values
     @set_dirty_and_flush_changes
     def set_value(self, section, option, value):
@@ -485,7 +550,29 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         :param option: Name of the options whose value to set
 
         :param value: Value to set the option to. It must be a string or convertible
-            to a string"""
+            to a string
+        :return: this instance"""
         if not self.has_section(section):
             self.add_section(section)
-        self.set(section, option, str(value))
+        self.set(section, option, self._value_to_string(value))
+        return self
+
+    def rename_section(self, section, new_name):
+        """rename the given section to new_name
+        :raise ValueError: if section doesn't exit
+        :raise ValueError: if a section with new_name does already exist
+        :return: this instance
+        """
+        if not self.has_section(section):
+            raise ValueError("Source section '%s' doesn't exist" % section)
+        if self.has_section(new_name):
+            raise ValueError("Destination section '%s' already exists" % new_name)
+
+        super(GitConfigParser, self).add_section(new_name)
+        for k, v in self.items(section):
+            self.set(new_name, k, self._value_to_string(v))
+        # end for each value to copy
+
+        # This call writes back the changes, which is why we don't have the respective decorator
+        self.remove_section(section)
+        return self
