@@ -12,7 +12,10 @@ from git.test.lib import (
     with_rw_repo
 )
 from git.util import Actor
-from git.exc import HookExecutionError
+from git.exc import (
+    HookExecutionError,
+    InvalidGitRepositoryError
+)
 from git import (
     IndexFile,
     BlobFilter,
@@ -381,12 +384,13 @@ class TestIndex(TestBase):
         num_entries = len(index.entries)
         cur_head = rw_repo.head
 
-        uname = "Some Developer"
+        uname = u"Thomas Müller"
         umail = "sd@company.com"
         writer = rw_repo.config_writer()
         writer.set_value("user", "name", uname)
         writer.set_value("user", "email", umail)
         writer.release()
+        assert writer.get_value("user", "name") == uname
 
         # remove all of the files, provide a wild mix of paths, BaseIndexEntries,
         # IndexEntries
@@ -739,7 +743,8 @@ class TestIndex(TestBase):
         # property rw_bare_repo.working_tree_dir will return '/tmp'
         # instead of throwing the Exception we are expecting. This is
         # a quick hack to make this test fail when expected.
-        rw_bare_repo._working_tree_dir = None
+        assert rw_bare_repo.working_tree_dir is None
+        assert rw_bare_repo.bare
         contents = b'This is a BytesIO file'
         filesize = len(contents)
         fileobj = BytesIO(contents)
@@ -757,6 +762,6 @@ class TestIndex(TestBase):
         path = os.path.join('git', 'test', 'test_index.py')
         try:
             rw_bare_repo.index.add([path])
-        except Exception as e:
-            asserted = "does not have a working tree" in str(e)
+        except InvalidGitRepositoryError:
+            asserted = True
         assert asserted, "Adding using a filename is not correctly asserted."
